@@ -41,7 +41,7 @@ from sonusai import update_console_handler
 from sonusai.metrics import calculate_optimal_thresholds
 from sonusai.metrics import class_summary
 from sonusai.metrics import snr_summary
-from sonusai.queries.get_organized_snrs import get_mixids_from_ordered_snr
+from sonusai.queries.queries import get_mixids_from_snr
 from sonusai.utils import human_readable_size
 from sonusai.utils import read_predict_data
 from sonusai.utils import reshape_truth_predict
@@ -132,7 +132,7 @@ def evaluate(mixdb: dict,
     else:
         logger.info(f'Mode:  Multi-label / Binary')
 
-    mxid_snro = get_mixids_from_ordered_snr(mixdb=mixdb)
+    mxid_snro = get_mixids_from_snr(mixdb=mixdb)
     snrlist = list(mxid_snro.keys())
     snrlist.sort(reverse=True)
     logger.info(f'Ordered SNRs: {snrlist}\n')
