@@ -63,7 +63,7 @@ from sonusai.mixture import truth_reduction
 from sonusai.utils import grouper
 from sonusai.utils import human_readable_size
 from sonusai.utils import int16_to_float
-from sonusai.utils import p_map
+from sonusai.utils import p_tqdm_map
 from sonusai.utils import seconds_to_hms
 from sonusai.utils import to_numpy_array
 from sonusai.utils import trim_docstring
@@ -97,8 +97,10 @@ def init(mixdb: dict,
 
     if logging:
         logger.info('')
-        logger.info(f'Found {len(mixdb_out["mixtures"])} mixtures to process')
-        logger.info(f'{total_samples} samples, {transform_frames} transform frames, {feature_frames} feature frames')
+        logger.info(f'Found {len(mixdb_out["mixtures"]):,} mixtures to process')
+        logger.info(f'{total_samples:,} samples, '
+                    f'{transform_frames:,} transform frames, '
+                    f'{feature_frames:,} feature frames')
 
     return mixdb_out, total_samples, transform_frames, feature_frames
 
@@ -150,11 +152,11 @@ def genft(mixdb: dict,
         progress = tqdm(total=len(mixdb_out['mixtures']), desc='genft', disable=not show_progress)
         progress_needs_close = True
 
-    p_map(_process_mixture,
-          list(range(len(mixdb_out['mixtures']))),
-          progress=progress,
-          initializer=mp_init,
-          initargs=(mp_feature, mp_truth_f, mp_segsnr))
+    p_tqdm_map(_process_mixture,
+               list(range(len(mixdb_out['mixtures']))),
+               progress=progress,
+               initializer=mp_init,
+               initargs=(mp_feature, mp_truth_f, mp_segsnr))
 
     if progress_needs_close:
         progress.close()
@@ -250,6 +252,7 @@ def main():
         update_console_handler(verbose)
         initial_log_messages('genft')
 
+        logger.info(f'\nLoad mixture database from {mixdb_name}')
         mixdb = load_mixdb(name=mixdb_name)
         mixid = load_mixid(name=mixid_name, mixdb=mixdb)
 
