@@ -61,7 +61,7 @@ from sonusai.mixture import new_mixdb_from_mixid
 from sonusai.mixture import set_mixture_offsets
 from sonusai.utils import grouper
 from sonusai.utils import human_readable_size
-from sonusai.utils import p_map
+from sonusai.utils import p_tqdm_map
 from sonusai.utils import seconds_to_hms
 from sonusai.utils import to_numpy_array
 from sonusai.utils import trim_docstring
@@ -99,8 +99,8 @@ def init(mixdb: dict,
 
     if logging:
         logger.info('')
-        logger.info(f'Found {len(mixdb_out["mixtures"])} mixtures to process')
-        logger.info(f'{total_samples} samples')
+        logger.info(f'Found {len(mixdb_out["mixtures"]):,} mixtures to process')
+        logger.info(f'{total_samples:,} samples')
 
     return mixdb_out, total_samples
 
@@ -161,11 +161,11 @@ def genmix(mixdb: dict,
         progress = tqdm(total=len(mixdb_out['mixtures']), desc='genmix', disable=not show_progress)
         progress_needs_close = True
 
-    p_map(_process_mixture,
-          list(range(len(mixdb_out['mixtures']))),
-          progress=progress,
-          initializer=mp_init,
-          initargs=(mp_mixture, mp_truth_t, mp_target, mp_noise, mp_segsnr))
+    p_tqdm_map(_process_mixture,
+               list(range(len(mixdb_out['mixtures']))),
+               progress=progress,
+               initializer=mp_init,
+               initargs=(mp_mixture, mp_truth_t, mp_target, mp_noise, mp_segsnr))
 
     if progress_needs_close:
         progress.close()
@@ -244,6 +244,7 @@ def main():
         update_console_handler(verbose)
         initial_log_messages('genmix')
 
+        logger.info(f'\nLoad mixture database from {mixdb_name}')
         mixdb = load_mixdb(name=mixdb_name)
         mixid = load_mixid(mixdb=mixdb, name=mixid_name)
 
