@@ -54,8 +54,6 @@ noise_mix:
   snrs:
     - 20
 
-target_count: 5
-
 feature: gfr32ts2
 
 output: data/my_mix.h5
@@ -91,9 +89,7 @@ The mixture is created with potential data augmentation functions in the followi
 4. add augmented noise to augmented target
 
 The mixture length is the target length by default, and the noise signal is repeated
-if it is shorter, or trimmed if longer. If 'target_count: <count>' is specified, then
-the target audio is concatenated <count> times for each augmentation rule.
-(See the Augmentation section for details on augmentation rules.)
+if it is shorter, or trimmed if longer.
 
 #### Target and noise using path lists
 
@@ -117,22 +113,11 @@ will find all .wav files in the specified directories and process them as target
 """
 import json
 import time
-from glob import glob
-from os import listdir
-from os.path import dirname
-from os.path import isabs
-from os.path import isdir
-from os.path import join
 from os.path import splitext
 from random import seed
-from typing import List
-from typing import Union
 
-import numpy as np
-import sox
 import yaml
 from docopt import docopt
-from pyaaware import FeatureGenerator
 from tqdm import tqdm
 
 import sonusai
@@ -143,20 +128,19 @@ from sonusai import logger
 from sonusai import update_console_handler
 from sonusai.mixture import apply_augmentation
 from sonusai.mixture import estimate_audio_length
-from sonusai.mixture import generate_truth
 from sonusai.mixture import get_augmentations
-from sonusai.mixture import get_class_count
 from sonusai.mixture import get_class_weights_threshold
-from sonusai.mixture import get_next_noise
+from sonusai.mixture import get_feature_stats
+from sonusai.mixture import get_input_files
+from sonusai.mixture import get_max_class
 from sonusai.mixture import get_total_class_count
 from sonusai.mixture import load_config
+from sonusai.mixture import log_duration_and_sizes
+from sonusai.mixture import process_mixture
 from sonusai.mixture import process_target
-from sonusai.mixture import raw_load_config
-from sonusai.mixture import read_audio
+from sonusai.mixture import read_all_audio
 from sonusai.mixture import update_truth_settings
-from sonusai.utils import expandvars
-from sonusai.utils import human_readable_size
-from sonusai.utils import p_map
+from sonusai.utils import p_tqdm_map
 from sonusai.utils import seconds_to_hms
 from sonusai.utils import trim_docstring
 
@@ -222,39 +206,59 @@ def genmixdb(file: str = None,
         logger.debug(f'SNRs: {config["noise_mix"]["snrs"]}\n')
         logger.debug(f'Exhaustive noise: {config["noise_mix"]["exhaustive"]}\n')
 
+    if config['truth_mode'] not in ['normal', 'mutex']:
+        raise SonusAIError(f'invalid truth_mode: {config["truth_mode"]}')
+    truth_mutex = config['truth_mode'] == 'mutex'
+    max_class = get_max_class(config['num_classes'], truth_mutex)
+
+    (feature_ms,
+     feature_samples,
+     feature_step_ms,
+     feature_step_samples,
+     num_bands,
+     stride) = get_feature_stats(feature_mode=config['feature'],
+                                 frame_size=config['frame_size'],
+                                 num_classes=config['num_classes'],
+                                 truth_mutex=truth_mutex)
+
+    mixdb = {
+        'class_balancing_augmentation': config['class_balancing_augmentation'],
+        'class_count':                  list(),
+        'class_labels':                 config['class_labels'],
+        'class_weights_threshold':      get_class_weights_threshold(config),
+        'feature':                      config['feature'],
+        'feature_samples':              feature_samples,
+        'feature_step_samples':         feature_step_samples,
+        'frame_size':                   config['frame_size'],
+        'mixtures':                     list(),
+        'noise_mix':                    {
+            'augmentations': noise_augmentations,
+            'exhaustive':    config['noise_mix']['exhaustive'],
+            'files':         noises,
+            'snrs':          config['noise_mix']['snrs'],
+        },
+        'num_classes':                  config['num_classes'],
+        'seed':                         config['seed'],
+        'target_augmentations':         target_augmentations,
+        'targets':                      targets,
+        'truth_mutex':                  truth_mutex,
+        'truth_reduction_function':     config['truth_reduction_function'],
+        'truth_settings':               config['truth_settings'],
+    }
+
+    MP_DICT['mixdb'] = mixdb
+    MP_DICT['config'] = config
+
+    raw_noise_audio, raw_target_audio = read_all_audio(noises=noises, targets=targets, show_progress=show_progress)
+
+    MP_DICT['raw_target_audio'] = raw_target_audio
+
     noise_sets = len(noises) * len(noise_augmentations)
     target_sets = len(targets) * len(target_augmentations) * len(config['noise_mix']['snrs'])
     total_mixtures = noise_sets * target_sets
     if logging:
         logger.info('')
-        logger.info(f'Found {total_mixtures} mixtures to process')
-
-    class_weights_threshold = get_class_weights_threshold(config)
-
-    if config['truth_mode'] not in ['normal', 'mutex']:
-        raise SonusAIError(f'invalid truth_mode: {config["truth_mode"]}')
-
-    truth_mutex = config['truth_mode'] == 'mutex'
-    if truth_mutex:
-        max_class = config['num_classes'] - 1
-    else:
-        max_class = config['num_classes']
-
-    fg = FeatureGenerator(feature_mode=config['feature'],
-                          frame_size=config['frame_size'],
-                          num_classes=config['num_classes'],
-                          truth_mutex=truth_mutex)
-
-    num_bands = fg.num_bands
-    stride = fg.stride
-    step = fg.step
-    decimation = fg.decimation
-
-    transform_frame_ms = float(config['frame_size']) / float(sonusai.mixture.SAMPLE_RATE / 1000)
-    feature_ms = transform_frame_ms * decimation * stride
-    feature_step_ms = transform_frame_ms * decimation * step
-    feature_samples = config['frame_size'] * decimation * stride
-    feature_step_samples = config['frame_size'] * decimation * step
+        logger.info(f'Found {total_mixtures:,} mixtures to process')
 
     total_duration = 0
     for duration in [sub['duration'] for sub in targets]:
@@ -268,67 +272,17 @@ def genmixdb(file: str = None,
     total_duration *= noise_sets
     total_duration *= len(config['noise_mix']['snrs'])
 
-    total_samples = total_duration * sonusai.mixture.SAMPLE_RATE
-
-    mixture_bytes = total_samples * sonusai.mixture.BIT_DEPTH / 8
-    truth_t_bytes = total_samples * config['num_classes'] * sonusai.mixture.FLOAT_BYTES
-    feature_bytes = total_samples / feature_step_samples * stride * num_bands * sonusai.mixture.FLOAT_BYTES
-    truth_f_bytes = total_samples / feature_step_samples * config['num_classes'] * sonusai.mixture.FLOAT_BYTES
-
     if logging:
-        logger.info('')
-        logger.info(f'Estimated duration:   {seconds_to_hms(seconds=total_duration)}')
-        logger.info('Estimated sizes:')
-        logger.info(f' mixture:             {human_readable_size(mixture_bytes, 1)}')
-        logger.info(f' truth_t:             {human_readable_size(truth_t_bytes, 1)}')
-        logger.info(f' feature:             {human_readable_size(feature_bytes, 1)}')
-        logger.info(f' truth_f:             {human_readable_size(truth_f_bytes, 1)}')
+        log_duration_and_sizes(total_duration=total_duration,
+                               num_classes=config['num_classes'],
+                               feature_step_samples=feature_step_samples,
+                               num_bands=num_bands,
+                               stride=stride,
+                               desc='Estimated')
         logger.info(f'Feature shape:        {stride} x {num_bands} ({stride * num_bands} total params)')
         logger.info(f'Feature samples:      {feature_samples} samples ({feature_ms} ms)')
         logger.info(f'Feature step samples: {feature_step_samples} samples ({feature_step_ms} ms)')
 
-    mixdb = {
-        'class_count':                  list(),
-        'class_labels':                 config['class_labels'],
-        'class_weights_threshold':      list(class_weights_threshold),
-        'feature':                      config['feature'],
-        'feature_samples':              feature_samples,
-        'feature_step_samples':         feature_step_samples,
-        'frame_size':                   config['frame_size'],
-        'mixtures':                     list(),
-        'noise_mix':                    {
-            'files':         noises,
-            'augmentations': noise_augmentations,
-            'snrs':          config['noise_mix']['snrs'],
-            'exhaustive':    config['noise_mix']['exhaustive'],
-        },
-        'num_classes':                  config['num_classes'],
-        'target_augmentations':         target_augmentations,
-        'targets':                      targets,
-        'truth_mutex':                  truth_mutex,
-        'truth_reduction_function':     config['truth_reduction_function'],
-        'class_balancing_augmentation': config['class_balancing_augmentation'],
-        'seed':                         config['seed'],
-    }
-
-    MP_DICT['mixdb'] = mixdb
-    MP_DICT['config'] = config
-
-    # Read in all audio data beforehand to avoid reading it multiple times in the loop
-    if logging:
-        logger.debug('Reading noise audio')
-    raw_noise_audio = list()
-    for noise in noises:
-        raw_noise_audio.append(read_audio(name=noise['name']))
-
-    if logging:
-        logger.debug('Reading target audio')
-    raw_target_audio = list()
-    for target in targets:
-        raw_target_audio.append(read_audio(name=target['name']))
-
-    MP_DICT['raw_target_audio'] = raw_target_audio
-
     # Get indices and offsets
     mixtures = [[] for _ in range(noise_sets)]
     n_id = 0
@@ -379,8 +333,7 @@ def genmixdb(file: str = None,
             audio_in=raw_noise_audio[mixtures[n_id][0]['noise_file_index']],
             augmentation=noise_augmentations[mixtures[n_id][0]['noise_augmentation_index']],
             length_common_denominator=1)
-        mixtures[n_id] = p_map(_process_mixture, mixtures[n_id], progress=progress)
-
+        mixtures[n_id] = p_tqdm_map(_process_mixture, mixtures[n_id], progress=progress)
     progress.close()
 
     # Flatten mixtures
@@ -389,109 +342,17 @@ def genmixdb(file: str = None,
 
     total_samples = sum([sub['samples'] for sub in mixdb['mixtures']])
     total_duration = total_samples / sonusai.mixture.SAMPLE_RATE
-    mixture_bytes = total_samples * sonusai.mixture.BIT_DEPTH / 8
-    truth_t_bytes = total_samples * config['num_classes'] * 4
-    feature_bytes = total_samples / feature_step_samples * stride * num_bands * 4
-    truth_f_bytes = total_samples / feature_step_samples * config['num_classes'] * 4
     if logging:
-        logger.info('')
-        logger.info(f'Actual duration: {seconds_to_hms(seconds=total_duration)}')
-        logger.info('Actual sizes:')
-        logger.info(f' mixture:        {human_readable_size(mixture_bytes, 1)}')
-        logger.info(f' truth_t:        {human_readable_size(truth_t_bytes, 1)}')
-        logger.info(f' feature:        {human_readable_size(feature_bytes, 1)}')
-        logger.info(f' truth_f:        {human_readable_size(truth_f_bytes, 1)}')
+        log_duration_and_sizes(total_duration=total_duration,
+                               num_classes=mixdb['num_classes'],
+                               feature_step_samples=feature_step_samples,
+                               num_bands=num_bands,
+                               stride=stride,
+                               desc='Actual')
 
     return mixdb
 
 
-def get_input_files(records: List[dict],
-                    truth_settings: Union[List[dict], None] = None) -> List[dict]:
-    if truth_settings is None:
-        truth_settings = list()
-
-    files = list()
-    for record in records:
-        append_input_files(files, record, truth_settings)
-    return files
-
-
-def append_input_files(files: List[dict],
-                       in_record: Union[dict, str],
-                       truth_settings: List[dict],
-                       tokens: Union[dict, None] = None) -> None:
-    if tokens is None:
-        tokens = dict()
-
-    if isinstance(in_record, dict):
-        if 'target_name' in in_record:
-            in_name = in_record['target_name']
-        else:
-            raise SonusAIError('Target list contained record without target_name')
-
-        if 'truth_settings' in in_record:
-            truth_settings = in_record['truth_settings']
-    else:
-        in_name = in_record
-
-    in_name, new_tokens = expandvars(in_name)
-    tokens.update(new_tokens)
-    names = glob(in_name)
-    if not names:
-        raise SonusAIError(f'Could not find {in_name}. Make sure path exists')
-    for name in names:
-        ext = splitext(name)[1].lower()
-        dir_name = dirname(name)
-        if isdir(name):
-            for file in listdir(name):
-                child = file
-                if not isabs(child):
-                    child = join(dir_name, child)
-                append_input_files(files, child, truth_settings, tokens)
-        else:
-            try:
-                if ext == '.txt':
-                    with open(file=name, mode='r') as txt_file:
-                        for line in txt_file:
-                            # strip comments
-                            child = line.partition('#')[0]
-                            child = child.rstrip()
-                            if child:
-                                child, new_tokens = expandvars(child)
-                                tokens.update(new_tokens)
-                                if not isabs(child):
-                                    child = join(dir_name, child)
-                                append_input_files(files, child, truth_settings, tokens)
-                elif ext == '.yml':
-                    try:
-                        yml_config = raw_load_config(name)
-
-                        if 'targets' in yml_config:
-                            for record in yml_config['targets']:
-                                append_input_files(files, record, truth_settings, tokens)
-                    except Exception as e:
-                        raise SonusAIError(f'Error processing {name}: {e}')
-                else:
-                    sox.file_info.validate_input_file(name)
-                    duration = sox.file_info.duration(name)
-                    for key, value in tokens.items():
-                        name = name.replace(value, f'${key}')
-                    entry = {
-                        'name':     name,
-                        'duration': duration,
-                    }
-                    if len(truth_settings) > 0:
-                        entry['truth_settings'] = truth_settings
-                        for truth_setting in entry['truth_settings']:
-                            if 'function' in truth_setting and truth_setting['function'] == 'file':
-                                truth_setting['config']['file'] = splitext(name)[0] + '.h5'
-                    files.append(entry)
-            except SonusAIError:
-                raise
-            except Exception as e:
-                raise SonusAIError(f'Error processing {name}: {e}')
-
-
 def get_output_from_config(config: dict, config_name: str) -> str:
     try:
         config_base = splitext(config_name)[0]
@@ -507,82 +368,10 @@ def _process_target(record: dict) -> dict:
 
 
 def _process_mixture(record: dict) -> dict:
-    mixdb = MP_DICT['mixdb']
-    raw_target_audio = MP_DICT['raw_target_audio']
-    augmented_noise_audio = MP_DICT['augmented_noise_audio']
-
-    target_index = record['target_file_index']
-    noise_offset = record['noise_offset']
-    target_augmentation_index = record['target_augmentation_index']
-
-    target_augmentation = mixdb['target_augmentations'][target_augmentation_index]
-
-    augmented_target_audio = apply_augmentation(audio_in=raw_target_audio[target_index],
-                                                augmentation=target_augmentation,
-                                                length_common_denominator=mixdb['feature_step_samples'])
-
-    record['samples'] = len(augmented_target_audio)
-
-    # target_gain is used to back out the gain augmentation in order to return the target audio
-    # to its normalized level when calculating truth.
-    if 'gain' in target_augmentation:
-        record['target_gain'] = 10 ** (target_augmentation['gain'] / 20)
-    else:
-        record['target_gain'] = 1
-
-    truth_index = [sub['index'] for sub in mixdb['targets'][target_index]['truth_settings']]
-
-    truth = generate_truth(mixdb=mixdb,
-                           record=record,
-                           target=augmented_target_audio)
-
-    class_weights_threshold = get_class_weights_threshold(mixdb)
-
-    record['class_count'] = get_class_count(truth_index=truth_index,
-                                            truth=truth,
-                                            class_weights_threshold=class_weights_threshold)
-
-    noise_segment, _ = get_next_noise(offset_in=noise_offset,
-                                      length=len(augmented_target_audio),
-                                      audio_in=augmented_noise_audio)
-
-    if record['snr'] < -96:
-        # Special case for zeroing out target data
-        record['target_snr_gain'] = 0
-        record['noise_snr_gain'] = 1
-        record['class_count'] = [[0] * len(inner) for inner in record['class_count']]
-        # Setting target_gain to zero will cause the truth to be all zeros.
-        record['target_gain'] = 0
-    elif record['snr'] > 96:
-        # Special case for zeroing out noise data
-        record['target_snr_gain'] = 1
-        record['noise_snr_gain'] = 0
-    else:
-        target_energy = np.mean(np.square(np.single(augmented_target_audio)))
-        noise_energy = np.mean(np.square(np.single(noise_segment)))
-        noise_gain = np.sqrt(target_energy / noise_energy) / 10 ** (
-                record['snr'] / 20)
-
-        # Check for noise_gain > 1 to avoid clipping
-        if noise_gain > 1:
-            record['target_snr_gain'] = 1 / noise_gain
-            record['noise_snr_gain'] = 1
-        else:
-            record['target_snr_gain'] = 1
-            record['noise_snr_gain'] = noise_gain
-
-    # Check for clipping in mixture
-    gain_adjusted_target_audio = np.single(augmented_target_audio) * record['target_snr_gain']
-    gain_adjusted_noise_audio = np.single(noise_segment) * record['noise_snr_gain']
-    mixture_audio = gain_adjusted_target_audio + gain_adjusted_noise_audio
-
-    if any(abs(mixture_audio) >= 32768):
-        # Clipping occurred; lower gains to bring audio within int16 bounds
-        gain_adjustment = 32760 / max(abs(mixture_audio))
-        record['target_snr_gain'] *= gain_adjustment
-        record['noise_snr_gain'] *= gain_adjustment
-
-    return record
+    return process_mixture(record=record,
+                           mixdb=MP_DICT['mixdb'],
+                           raw_target_audio=MP_DICT['raw_target_audio'],
+                           augmented_noise_audio=MP_DICT['augmented_noise_audio'])
 
 
 def main():
