@@ -3,6 +3,7 @@
 usage: sonusai [--version] [--help] <command> [<args>...]
 
 The sonusai commands are:
+   cbmixdb      Class balance a mixture database
    evaluate     Evaluate model performance
    genft        Generate feature and truth data
    genmix       Generate mixture and truth data
@@ -29,6 +30,7 @@ from sonusai.utils import trim_docstring
 def main():
     try:
         commands = (
+            'cbmixdb',
             'evaluate',
             'genft',
             'genmix',
