@@ -7,7 +7,7 @@ import pandas as pd
 
 from sonusai.metrics.one_hot import one_hot
 from sonusai.mixture.get_mixid_data import get_mixid_data
-from sonusai.queries.get_organized_snrs import get_mixids_from_ordered_snr
+from sonusai.queries.queries import get_mixids_from_snr
 
 
 def snr_summary(mixdb: dict,
@@ -38,7 +38,7 @@ def snr_summary(mixdb: dict,
     """
     num_classes = truth_f.shape[1]
 
-    mxid_snro = get_mixids_from_ordered_snr(mixdb=mixdb, mixid=mixid)
+    mxid_snro = get_mixids_from_snr(mixdb=mixdb, mixid=mixid)
 
     # Check pred_thr array or scalar and return final scalar pred_thr value
     if not mixdb['truth_mutex'] and num_classes > 1:
