@@ -26,6 +26,15 @@ def get_mixid_data(mixdb: dict,
     total_frames = sum([file_frame_segments[m].length for m in file_frame_segments])
     ytrue = np.empty((total_frames, num_classes), dtype=np.single)
     ypred = np.empty((total_frames, dnum), dtype=np.single)
+
+    # Handle the special case when input data is smaller, i.e., prediction when total mixture
+    # length is a non-multiple of the batch size. In this case just pad both with zeros; should
+    # have negligible effect on metrics
+    fdiff = total_frames - truth_f.shape[0]
+    if fdiff > 0:
+        truth_f = np.concatenate((truth_f, np.zeros((fdiff, num_classes))))
+        predict = np.concatenate((predict, np.zeros((fdiff, num_classes))))
+
     start = 0
     for m in file_frame_segments:
         length = file_frame_segments[m].length
