@@ -1,16 +1,20 @@
+from typing import List
+
 import numpy as np
 
+from sonusai import SonusAIError
+from sonusai.mixture.apply_augmentation import apply_augmentation
+from sonusai.mixture.audio_db import get_noise_audio_from_db
+from sonusai.mixture.audio_db import get_target_audio_from_db
+from sonusai.mixture.generate_segsnr import generate_segsnr
+from sonusai.mixture.generate_truth import generate_truth
+from sonusai.mixture.get_next_noise import get_next_noise
+
 
 def get_target_noise_audio(mixdb: dict,
                            record: dict,
-                           target_audios: list,
-                           noise_audios: list) -> (np.ndarray, np.ndarray):
-    from sonusai import SonusAIError
-    from sonusai.mixture.apply_augmentation import apply_augmentation
-    from sonusai.mixture.audio_db import get_noise_audio_from_db
-    from sonusai.mixture.audio_db import get_target_audio_from_db
-    from sonusai.mixture.get_next_noise import get_next_noise
-
+                           target_audios: List[dict],
+                           noise_audios: List[dict]) -> (np.ndarray, np.ndarray):
     if record['samples'] % mixdb['frame_size'] != 0:
         raise SonusAIError(f'Number of samples in record is not a multiple of {mixdb["frame_size"]}')
     target_file_index = record['target_file_index']
@@ -36,14 +40,11 @@ def get_target_noise_audio(mixdb: dict,
 
 def get_audio_and_truth_t(mixdb: dict,
                           record: dict,
-                          target_audios: list,
-                          noise_audios: list,
+                          target_audios: List[dict],
+                          noise_audios: List[dict],
                           compute_truth: bool = True,
                           compute_segsnr: bool = False,
                           frame_based_segnsr: bool = False) -> (np.ndarray, np.ndarray, np.ndarray, np.ndarray):
-    from sonusai.mixture.generate_segsnr import generate_segsnr
-    from sonusai.mixture.generate_truth import generate_truth
-
     target, noise = get_target_noise_audio(mixdb=mixdb,
                                            record=record,
                                            target_audios=target_audios,
@@ -54,7 +55,6 @@ def get_audio_and_truth_t(mixdb: dict,
     truth_t = generate_truth(mixdb=mixdb,
                              record=record,
                              target=target,
-                             noise=noise,
                              compute=compute_truth)
 
     segsnr = generate_segsnr(mixdb=mixdb,
