@@ -1,4 +1,4 @@
-"""mkwav
+"""sonusai mkwav
 
 usage: mkwav [-hvn] FILE...
 
@@ -41,18 +41,17 @@ def mkwav(files: list, dry_run: bool = False, verbose: bool = False):
             logger.info(f'Processing {file}')
         try:
             with h5py.File(file, 'r') as f:
-                keys = f.keys()
                 for audio_name in audio_names:
-                    if audio_name in keys:
+                    if audio_name in f:
                         out_name = base_name + '_' + audio_name + '.wav'
                         audio = f['/' + audio_name][:]
                         if verbose:
                             logger.info(f'Writing {len(audio)} samples to {out_name}')
                         if not dry_run:
-                            with wave.open(out_name, 'w') as w:
-                                w.setnchannels(mixture.channel_count)
-                                w.setsampwidth(mixture.sample_bytes)
-                                w.setframerate(mixture.sample_rate)
+                            with wave.open(out_name, mode='w') as w:
+                                w.setnchannels(mixture.CHANNEL_COUNT)
+                                w.setsampwidth(mixture.SAMPLE_BYTES)
+                                w.setframerate(mixture.SAMPLE_RATE)
                                 w.writeframesraw(audio)
         except Exception as e:
             logger.error(f'Error processing {file}: {e}')
