@@ -1,9 +1,5 @@
 # SonusAI query utilities
 
-from sonusai.queries.get_organized_snrs import get_mixids_from_ordered_snr
-from sonusai.queries.get_organized_snrs import get_mixids_from_random_snr
-from sonusai.queries.get_organized_snrs import get_ordered_snrs
-from sonusai.queries.get_organized_snrs import get_random_snrs
 from sonusai.queries.queries import get_mixids_from_noise
 from sonusai.queries.queries import get_mixids_from_noise_augmentation
 from sonusai.queries.queries import get_mixids_from_snr
