@@ -7,8 +7,10 @@ from sonusai.utils.int16_to_float import int16_to_float
 from sonusai.utils.onnx_utils import add_sonusai_metadata
 from sonusai.utils.onnx_utils import create_onnx_from_keras
 from sonusai.utils.onnx_utils import replace_stateful_grus
-from sonusai.utils.p_tqdm import p_map
-from sonusai.utils.p_tqdm import p_umap
+from sonusai.utils.parallel import p_map
+from sonusai.utils.parallel import p_umap
+from sonusai.utils.parallel_tqdm import p_tqdm_map
+from sonusai.utils.parallel_tqdm import p_tqdm_umap
 from sonusai.utils.read_feature_data import read_feature_data
 from sonusai.utils.read_mixture_data import read_mixture_data
 from sonusai.utils.read_predict_data import read_predict_data
